#include <TinyWireS84.h>
#include "RunningMedian.h"

#define led_pin0 7
#define led_pin1 8
#define led_pin2 9
#define led_pin3 10

#define detector_pin0 0
#define detector_pin1 1
#define detector_pin2 2
#define detector_pin3 3

#define i2c_address 0xB

struct sensor_data_t{
  unsigned sensor0 : 10;
  unsigned sensor1 : 10;
  unsigned sensor2 : 10;
  unsigned sensor3 : 10;
};

void setup()
{
  pinMode(led_pin0, OUTPUT);
  pinMode(led_pin1, OUTPUT);
  pinMode(led_pin2, OUTPUT);
  pinMode(led_pin3, OUTPUT);
  
  TinyWireS.begin(i2c_address);
  TinyWireS.onRequest(on_request);
}

RunningMedian ir_readings[4];
RunningMedian dist_readings[4];

void readSensors(void)
{
  
  int preVal = analogRead(detector_pin0);
  ir_readings[0].add(preVal);
  digitalWrite(led_pin0, HIGH);
  delay(1);
  int newVal = analogRead(detector_pin0);
  if(preVal < newVal) 
    dist_readings[0].add((newVal-preVal));
  digitalWrite(led_pin0, LOW);

  
  preVal = analogRead(detector_pin1);
  ir_readings[1].add(preVal);
  digitalWrite(led_pin1, HIGH);
  delay(1);
  newVal = analogRead(detector_pin1);
  if(preVal < newVal) 
    dist_readings[1].add((newVal-preVal));
  digitalWrite(led_pin1, LOW);
  
  preVal = analogRead(detector_pin2);
  ir_readings[2].add(preVal);
  digitalWrite(led_pin2, HIGH);
  delay(1);
  newVal = analogRead(detector_pin2);
  if(preVal < newVal) 
    dist_readings[2].add((newVal-preVal));
  digitalWrite(led_pin2, LOW);
  
  preVal = analogRead(detector_pin3);
  ir_readings[3].add(preVal);
  digitalWrite(led_pin3, HIGH);
  delay(1);
  newVal = analogRead(detector_pin3);
  if(preVal < newVal) 
    dist_readings[3].add((newVal-preVal));
  digitalWrite(led_pin3, LOW);
}

unsigned long timestamp_high[4];
unsigned long timestamp_low[4];
int last_state[4];

int high_time[4];
int low_time[4];

void sendData(void)
{
  for(int i = 0; i < 4; i++)
  {
    if(millis() - timestamp_high[i] > 1000)
      high_time[i] = 0;
    if(millis() - timestamp_low[i] > 1000)
      high_time[i] = 0;
  }
  
  sensor_data_t high;
  high.sensor0 = high_time[0]; //ir_readings[0].getMedian();
  high.sensor1 = high_time[1]; //ir_readings[1].getMedian();
  high.sensor2 = high_time[2]; //ir_readings[2].getMedian();
  high.sensor3 = high_time[3]; //ir_readings[3].getMedian();

  sensor_data_t low;
  low.sensor0 = low_time[0]; //dist_readings[0].getMedian();
  low.sensor1 = low_time[1]; //dist_readings[1].getMedian();
  low.sensor2 = low_time[2]; //dist_readings[2].getMedian();
  low.sensor3 = low_time[3]; //dist_readings[3].getMedian();

  uint8_t* dist = (uint8_t*)&low;
  uint8_t* ir = (uint8_t*)&high;

  TinyWireS.send(dist[0]);
  TinyWireS.send(dist[1]);
  TinyWireS.send(dist[2]);
  TinyWireS.send(dist[3]);
  TinyWireS.send(dist[4]);

  TinyWireS.send(ir[0]);
  TinyWireS.send(ir[1]);
  TinyWireS.send(ir[2]);
  TinyWireS.send(ir[3]);
  TinyWireS.send(ir[4]);
}

void pulse(int delays)
{
  digitalWrite(5, HIGH);
  delay(delays);
  digitalWrite(5, LOW);
  delay(delays);
}


void loop()
{
  for(int i = 0; i < 4; i++)
  {
    if(analogRead(i) > 200)
    {
      if(last_state[i]) 
        return; //remained high
      //changed to high
      last_state[i] = true;
      timestamp_high[i] = millis();
      low_time[i] = millis() - timestamp_low[i];
    }
    else
    {
      if(!last_state[i]) 
        return; //remained low
      //changed to low
      last_state[i] = false;
      timestamp_low[i] = millis();
      high_time[i] = millis() - timestamp_high[i];
    }
  }
  
    
  //high0 = analogRead(detector_pin0);
  //high1 = analogRead(detector_pin1);
  //high2 = analogRead(detector_pin2);
  //high3 = analogRead(detector_pin3);
}

void on_request(void)
{
  //readSensors();
  //digitalWrite(5, HIGH);
  sendData();
}



