#include <Wire.h>
#include <Chirp.h>  

Chirp chirp;
double current_heading = 0.0;
double x = 0.0;
double y = 0.0;

void set_heading(double heading)
{
  if(heading > 2*PI)
  {
    current_heading = heading - 2*PI;
  }
  else if(heading < 0.0)
  {
    current_heading = heading + 2*PI;
  }
  else
  {
    current_heading = heading;
  }
}

void update_heading(double delta)
{
  set_heading(current_heading + delta);
}

void update_position(int steps)
{
  x += steps * cos(current_heading);
  y += steps * sin(current_heading);
}

double rad(double deg)
{
  return (deg * 71) / 4068;
}

double deg(double rad)
{
  return (rad * 4068) / 71;
}

double angle()
{
  return atan2(y, x);
}

double distance()
{
  return sqrt(x*x + y*y);
}

void turn_to_heading(double heading)
{
  int a = heading  - deg(current_heading);
  a = (a+180) % 360 - 180;
  chirp.turnDegrees(a, 100, true);
  update_heading(rad(a));
}

void debug()
{
  Serial.print("d: ");
  Serial.print(distance());
  Serial.print(" a: ");
  Serial.print(360 - deg(angle()));
  Serial.print(" h: ");
  Serial.print(deg(current_heading));
  Serial.print(", x: ");
  Serial.print(x);
  Serial.print(", y: ");
  Serial.println(y);
}

void move(int steps)
{
  chirp.moveSteps(steps, true);
  update_position(steps);
  debug();
}

void turn(double deg)
{
  chirp.turnDegrees(deg, 100, true);
  update_heading(rad(deg));
  debug();
}

void go_home()
{
  turn_to_heading(180 + deg(angle()));
  debug();
  chirp.moveSteps(distance(), true);
  update_position(distance());
  debug();
}

int random_sign()
{
  return random(1) == 1 ? -1 : 1;
}

void setup() {
  chirp.init();
} 



void loop() { 
  delay(5000); 
  int MANEUVERS = 8;
  int STEPS = MANEUVERS * 1000;
  int DEGREES = MANEUVERS * 120;
  x = 0;
  y = 0;
  set_heading(0.0);
  //Your code goes here:
  Serial.println("hallo");
  debug();
  while(MANEUVERS > 0)
  {
    int st = STEPS / MANEUVERS;
    int de = DEGREES / MANEUVERS;
    de = random(de*0.5, de*1.5);
    Serial.println(random_sign()*de);
    turn(random_sign() * de);
    move(st);
    STEPS -= st;
    DEGREES -= de;
    MANEUVERS--;    
    
  }
  go_home();
  Serial.println("finished");
  delay(7000);
  chirp.update();
}
